"""
Created on 2022-10-25

@author: wf
"""

import pprint

from basemkit.basetest import Basetest

from mwdocker.html_table import HtmlTables


class TestHtmlTables(Basetest):
    """
    test the HTML Tables parsere
    """

    def testHtmlTables(self):
        urls = [
            "https://wiki.bitplan.com/index.php/Special:Version",
            "https://en.wikipedia.org/wiki/Special:Version",
        ]
        for url in urls:
            html_table = HtmlTables(url)
            tables = html_table.get_tables("h2")
            pp = pprint.PrettyPrinter(indent=2)
            debug = self.debug
            # debug=True
            if debug:
                pp.pprint(tables)
            self.assertTrue("Installed extensions" in tables)
            pass



"""
Created on 2023-04-06

@author: wf
"""

import dataclasses
import json
import os
import re
import secrets
import socket
from dataclasses import dataclass, field
from pathlib import Path
from typing import List, Optional
from urllib.parse import urlparse

from basemkit.yamlable import lod_storable
from lodstorage.lod import LOD

from mwdocker.mw import ExtensionList
from mwdocker.docker_map import DockerMap

class Host:
    """
    Host name getter
    """

    @classmethod
    def get_default_host(cls) -> str:
        """
        Get the default host as a usable hostname or IP,
        never returning reverse-DNS PTRs and avoiding localhost which
        might cause to try socket access instead of proper host access
        """
        host = socket.getfqdn()

        # work around https://github.com/python/cpython/issues/79345
        if host == (
            "1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa"
        ):
            host = "127.0.0.1"

        elif host.endswith(".in-addr.arpa"):
            host = "127.0.0.1"

        elif host.endswith(".ip6.arpa"):
            host = "::1"

        elif host == "localhost":
            host = "127.0.0.1"

        return host



@lod_storable
class MwConfig:
    """
    MediaWiki and docker configuration for a Single Wiki
    """
    version: str = "1.39.13"
    smw_version: Optional[str] = None
    extensionNameList: Optional[List[str]] = field(
        default_factory=lambda: [
            "Admin Links",
            "Header Tabs",
            "ParserFunctions",
            "SyntaxHighlight",
            "Variables",
        ]
    )
    extensionJsonFile: Optional[str] = None
    user: str = "Sysop"
    prefix: str = "mw"
    logo: str = "$wgResourceBasePath/resources/assets/wiki.png"
    url = None
    full_url = None
    prot: str = "http"
    host: str = Host.get_default_host()
    article_path: Optional[str] = None  # "/index.php/$1"
    script_path: str = ""
    wikiId: Optional[str] = None
    # mysql settings
    mySQLRootPassword: Optional[str] = None
    mySQLPassword: Optional[str] = None
    mariaDBVersion: str = "11.4"

    # docker settings
    bind_mount: bool = False
    port: int = 9080
    base_port: Optional[int] = None
    sql_port: int = 9306
    container_base_name: Optional[str] = None
    # derived from container_base_name if different than default
    # an external db_container is going to be used
    db_container_name: Optional[str] = None
    networkName: str = "mwNetwork"
    docker_path: Optional[str] = None
    gid: int = 33  # www-data
    uid: int = 33  # www-data

    # build control
    verbose: bool = True
    random_password: bool = False
    force_user: bool = False
    lenient: bool = True
    password_length: int = 15
    forceRebuild: bool = False
    debug: bool = False
    # FIXME - we should avoid a predefined known password
    password: str = "sysop-1234!"

    def default_docker_path(self) -> str:
        """
        get the default docker path

        Returns:
            str: $HOME/.pymediawikidocker
        """
        home = str(Path.home())
        docker_path = f"{home}/.pymediawikidocker"
        return docker_path

    def __post_init__(self):
        """
        post initialization configuration
        """
        self.fullVersion = f"MediaWiki {self.version}"
        self.underscoreVersion = self.version.replace(".", "_")
        self.shortVersion = self.getShortVersion()
        if not self.docker_path:
            self.docker_path = self.default_docker_path()
        if not self.container_base_name:
            self.container_base_name = f"{self.prefix}-{self.shortVersion}"
        if not self.db_container_name:
            self.db_container_name = f"{self.container_base_name}-db"
        if not self.article_path:
            self.article_path = ""
        if not self.base_port:
            self.base_port = self.port
        self.reset_url(self.url)

    @property
    def has_external_db(self) -> bool:
        """
        Check if using external database container

        Returns:
            bool: True if db_container_name differs from default pattern
        """
        default_db_name = f"{self.container_base_name}-db"
        external= self.db_container_name != default_db_name
        return external

    def reset_url(self, url: str):
        """
        reset my url

        Args:
            url(str): the url to set
        """
        if url:
            pr = urlparse(url)
            self.prot = pr.scheme
            self.host = pr.hostname
            self.script_path = pr.path
            self.base_url = f"{self.prot}://{self.host}"
            self.full_url = url
        else:
            self.base_url = f"{self.prot}://{self.host}:{self.base_port}"

            self.full_url = f"{self.base_url}{self.script_path}"

    def reset_container_base_name(self, container_base_name: str=None):
        """
        reset the container base name to the given name

        Args:
            container_base_name(str): the new container base name
        """
        self.container_base_name = container_base_name
        self.__post_init__()

    def as_dict(self) -> dict:
        """
        return my fields as a dict
        dataclasses to dict conversion convenienc and information hiding

        Returns:
            dict: my fields in dict format
        """
        config_dict = dataclasses.asdict(self)
        return config_dict

    def as_json(self) -> str:
        """
        return me as a json string

        Returns:
            str: my json representation
        """
        config_dict = self.as_dict()
        json_str = json.dumps(config_dict, indent=2)
        return json_str

    def get_config_path(self) -> str:
        """
        get my configuration base path

        Returns:
            str: the path to my configuration
        """
        config_base_path = f"{self.docker_path}/{self.container_base_name}"
        os.makedirs(config_base_path, exist_ok=True)
        path = f"{config_base_path}/MwConfig.json"
        return path

    def save(self, path: str=None) -> str:
        """
        save my json

        Args:
            path(str): the path to store to - if None use {docker_path}/{container_base_name}/MwConfig.json
        Returns:
            str: the path
        """
        if path is None:
            path = self.get_config_path()

        json_str = self.as_json()
        with open(path, "w") as f:
            print(json_str, file=f)
        return path

    def load(self, path: str=None) -> "MwConfig":
        """
        load the the MwConfig from the given path of if path is None (default)
        use the config_path for the current configuration

        restores the ExtensionMap on load

        Args:
            path(str): the path to load from

        Returns:
            MwConfig: a MediaWiki Configuration
        """
        if path is None:
            path = self.get_config_path()
        with open(path, "r") as json_file:
            json_str = json_file.read()
            config = self.__class__.from_json(json_str)
            # restore extension map
            config.getExtensionMap(config.extensionNameList, config.extensionJsonFile)
            return config

    def getShortVersion(self, separator=""):
        """
        get my short version e.g. convert 1.27.7 to 127

        Returns:
            str: the short version string
        """
        versionMatch = re.match(r"(?P<major>[0-9]+)\.(?P<minor>[0-9]+)", self.version)
        shortVersion = (
            f"{versionMatch.group('major')}{separator}{versionMatch.group('minor')}"
        )
        return shortVersion

    def create_random_password(self, length: int=15) -> str:
        """
        create a random password

        Args:
            length(int): the length of the password

        Returns:
            str:a random password with the given length
        """
        random_password = secrets.token_urlsafe(length)
        return random_password

    def getWikiId(self):
        """
        get the wikiId

        Returns:
            str: e.g. mw-9080
        """
        if self.wikiId is None:
            wikiId = f"{self.prefix}-{self.port}"
        else:
            wikiId = self.wikiId
        return wikiId

    def getExtensionMap(
        self, extensionNameList: list=None, extensionJsonFile: str=None
    ):
        """
        get map of extensions to handle

        Args:
            extensionNameList (list): a list of extension names
            extensionJsonFile (str): the name of an extra extensionJsonFile (if any)
        """
        self.extensionMap = {}
        extensionList = ExtensionList.restore()
        self.extByName, duplicates = LOD.getLookup(extensionList.extensions, "name")
        if len(duplicates) > 0:
            print(f"{len(duplicates)} duplicate extensions: ")
            for duplicate in duplicates:
                print(duplicate.name)
        if extensionJsonFile is not None:
            extraExtensionList = ExtensionList.load_from_json_file(extensionJsonFile)  # @UndefinedVariable
            for ext in extraExtensionList.extensions:
                if ext.name in self.extByName:
                    print(f"overriding {ext.name} extension definition")
                self.extByName[ext.name] = ext
        if extensionNameList is not None:
            self.addExtensions(extensionNameList)
        return self.extensionMap

    def addExtensions(self, extensionNameList):
        """
        add extensions for the given list of extension names
        """
        for extensionName in extensionNameList:
            if extensionName in self.extByName:
                self.extensionMap[extensionName] = self.extByName[extensionName]
            else:
                print(f"warning: extension {extensionName} not known")

    def fromArgs(self, args):
        """
        initialize me from the given commmand line arguments

        Args:
            args(Namespace): the command line arguments
        """
        self.prefix = args.prefix
        self.article_path = args.article_path
        self.container_base_name = args.container_name
        self.db_container_name = args.db_container_name
        self.docker_path = args.docker_path
        self.extensionNameList = args.extensionNameList
        self.extensionJsonFile = args.extensionJsonFile
        self.bind_mount = args.bind_mount
        self.uid = args.uid
        self.gid = args.gid
        self.forceRebuild = args.forceRebuild or getattr(args, "force", False)
        self.host = args.host
        self.logo = args.logo
        self.mariaDBVersion = args.mariaDBVersion
        # passwords
        if args.mysqlRootPassword:
            self.mySQLRootPassword = args.mysqlRootPassword
        if not self.mySQLRootPassword:
            if args.db_container_name:
                # we need the password from the database container
                pass
            else:
                self.mySQLRootPassword = self.create_random_password(self.password_length)
        if args.mysqlPassword:
            self.mySQLPassword = args.mysqlPassword
        else:
            self.mySQLPassword = self.create_random_password(self.password_length)
        self.prot = args.prot
        self.script_path = args.script_path
        self.versions = args.versions
        self.user = args.user
        self.random_password = args.random_password
        self.force_user = args.force_user
        self.lenient = args.lenient
        self.password = args.password
        self.password_length = args.password_length
        self.base_port = args.base_port
        self.sql_port = args.sql_port
        self.smw_version = args.smw_version
        self.verbose = not args.quiet
        self.debug = args.debug
        self.getExtensionMap(self.extensionNameList, self.extensionJsonFile)
        self.reset_url(args.url)

    def addArgs(self, parser):
        """
        add Arguments to the given parser
        """
        parser.add_argument(
            "--article_path",
            default=self.article_path,
            help="change to any article_path you might need to set [default: %(default)s]",
        )
        parser.add_argument(
            "-bm",
            "--bind-mount",
            action="store_true",
            help="use bind mounts instead of volumes",
        )
        parser.add_argument(
            "-cn",
            "--container_name",
            default=self.container_base_name,
            help="set container name (only valid and recommended for single version call)",
        )
        parser.add_argument(
            "-dcn",
            "--db_container_name",
            help="set database container name [default: %(default)s]",
        )
        parser.add_argument(
            "-el",
            "--extensionList",
            dest="extensionNameList",
            nargs="*",
            default=self.extensionNameList,
            help="list of extensions to be installed [default: %(default)s]",
        )
        parser.add_argument(
            "-ej",
            "--extensionJson",
            dest="extensionJsonFile",
            default=self.extensionJsonFile,
            help="additional extension descriptions default: [default: %(default)s]",
        )
        # since -f is a default options we have to make sure we accept it as forceRebuild
        parser.add_argument(
            "--forceRebuild",
            action="store_true",
            default=self.forceRebuild,
            help="force rebuilding  [default: %(default)s]",
        )
        parser.add_argument(
            "-fu",
            "--force_user",
            action="store_true",
            default=self.force_user,
            help="force overwrite of wikiuser",
        )
        parser.add_argument(
            "--host",
            default=Host.get_default_host(),
            help="the host to serve / listen from [default: %(default)s]",
        )
        parser.add_argument(
            "-dp",
            "--docker_path",
            default=self.default_docker_path(),
            help="the base directory to store docker and jinja template files [default: %(default)s]",
        )
        parser.add_argument(
            "--lenient",
            action="store_true",
            help="do not throw error on wikiuser difference",
        )
        parser.add_argument(
            "--logo", default=self.logo, help="set Logo [default: %(default)s]"
        )
        parser.add_argument(
            "-mv",
            "--mariaDBVersion",
            dest="mariaDBVersion",
            default=self.mariaDBVersion,
            help="mariaDB Version to be installed [default: %(default)s]",
        )
        parser.add_argument(
            "--mysqlRootPassword",
            default=self.mySQLRootPassword,
            help="set sql root Password [default: %(default)s] - random password if None",
        )
        parser.add_argument(
            "--mysqlPassword",
            default=self.mySQLPassword,
            help="set sql user Password [default: %(default)s] - random password if None",
        )
        parser.add_argument(
            "-rp",
            "--random_password",
            action="store_true",
            default=self.random_password,
            help="create random password and create wikiuser while at it",
        )
        parser.add_argument(
            "-p",
            "--password",
            dest="password",
            default=self.password,
            help="set password for initial user [default: %(default)s] ",
        )
        parser.add_argument(
            "-pl",
            "--password_length",
            default=self.password_length,
            help="set the password length for random passwords[default: %(default)s] ",
        )
        parser.add_argument(
            "--prefix",
            default=self.prefix,
            help="the container name prefix to use [default: %(default)s]",
        )
        parser.add_argument(
            "--prot",
            default=self.prot,
            help="change to https in case [default: %(default)s]",
        )
        parser.add_argument(
            "--script_path",
            default=self.script_path,
            help="change to any script_path you might need to set [default: %(default)s]",
        )
        parser.add_argument(
            "--url",
            default=self.url,
            help="will set prot host,script_path, and optionally port based on the url given [default: %(default)s]",
        )
        parser.add_argument(
            "-sp",
            "--sql_base_port",
            dest="sql_port",
            type=int,
            default=self.sql_port,
            help="set base mySql port 3306 to be exposed - incrementing by one for each version [default: %(default)s]",
        )
        parser.add_argument(
            "-smw",
            "--smw_version",
            dest="smw_version",
            default=self.smw_version,
            help="set SemanticMediaWiki Version to be installed default is None - no installation of SMW",
        )
        parser.add_argument(
            "-u",
            "--user",
            dest="user",
            default=self.user,
            help="set username of initial user with sysop rights [default: %(default)s] ",
        )
        parser.add_argument("--uid", type=int, default=self.uid, help="User ID  (default: 33 for www-data)")
        parser.add_argument("--gid", type=int, default=self.gid, help="Group ID (default: 33 for www-data)")


@dataclass
class MwClusterConfig(MwConfig):
    """
    MediaWiki Cluster configuration for multiple wikis
    """

    versions: Optional[List[str]] = field(
        default_factory=lambda: ["1.35.13", "1.39.13", "1.43.3", "1.44.0"]
    )
    base_port: int = 9080

    def addArgs(self, parser):
        """
        add my arguments to the given parser
        """
        super().addArgs(parser)
        parser.add_argument(
            "-bp",
            "--base_port",
            dest="base_port",
            type=int,
            default=self.base_port,
            help="set how base html port 80 to be exposed - incrementing by one for each version [default: %(default)s]",
        )
        parser.add_argument(
            "-vl",
            "--version_list",
            dest="versions",
            nargs="*",
            default=self.versions,
            help="mediawiki versions to create docker applications for [default: %(default)s] ",
        )

    def fromArgs(self, args):
        """
        initialize me from the given commmand line arguments

        Args:
            args(Namespace): the command line arguments
        """
        dbc_name=args.db_container_name
        if dbc_name:
            env=DockerMap.getEnv(dbc_name)
            self.mySQLRootPassword=env["MYSQL_ROOT_PASSWORD"]
            pass
        super().fromArgs(args)



'''
Created on 2025-08-21

@author: wf
'''
from typing import Dict

from python_on_whales import docker
from python_on_whales.components.container.cli_wrapper import Container


class DockerMap:
    """
    helper class to convert lists of docker elements to maps for improved
    lookup functionality
    """

    _container_map = None

    @classmethod
    def getContainer(cls,container_name:str):
        containerMap = DockerMap.getContainerMap()
        if not container_name in containerMap:
            raise ValueError(f"container {container_name} is not a valid docker container")
        container=containerMap.get(container_name)
        return container

    @classmethod
    def getEnv(cls,container_name:str)->Dict[str,str]:
        container=cls.getContainer(container_name)
        env_dict = {}
        env_list=container.config.env
        for key_value in env_list:
            if '=' in key_value:
                key, value = key_value.split('=', 1)
                env_dict[key] = value
        return env_dict

    @staticmethod
    def getContainerMap(force_refresh: bool = True)->Dict[str,Container]:
        """
        get a cached map/dict of containers by container name

        Args:
            force_refresh: if True, refresh from docker instead of using cache
        """
        if DockerMap._container_map is None or force_refresh:
            DockerMap._container_map = {}
            for container in docker.container.list():
                DockerMap._container_map[container.name] = container
        return DockerMap._container_map


"""
Created on 2021-08-06
@author: wf
"""

from argparse import Namespace
import dataclasses
import sys

from mwdocker.config import MwClusterConfig
from mwdocker.docker import DockerApplication
from mwdocker.logger import Logger


class MediaWikiCluster(object):
    """
    a cluster of mediawiki docker Applications
    """

    # https://hub.docker.com/_/mediawiki
    # 2023-01-13
    # MediaWiki Extensions and Skins Security Release Supplement (1.35.9/1.38.4/1.39.1)
    # 2023-02-23 1.39.2 released
    # 2023-04-04 1.39.3 upgrade
    # 2023-10-04 1.39.5 upgrade
    # 2024-04-15 1.39.7 upgrade
    # 2024-08-02 1.39.8 upgrade
    # 2024-10-11 1.39.10 upgrade
    # 2025-03-18 1.39.11 upgrade 1.43.0 addition
    # 2025-04-13 Security and maintenance release: 1.39.12 / 1.42.6 / 1.43.1
    # 2025-06-30 Security and maintenance release: 1.39.13 / 1.42.7 / 1.43.3

    def __init__(self, config: MwClusterConfig,args:Namespace=None):
        """
        Constructor

        Args:
            config(MWClusterConfig): the MediaWiki Cluster Configuration to use
        """
        self.config = config
        self.args=args
        self.apps = {}

    def createApps(self, withGenerate: bool = True) -> dict:
        """
        create my apps

        Args:
            withGenerate(bool): if True generate the config files

        Returns:
            dict(str): a dict of apps by version
        """
        exitCode = self.checkDocker()
        if exitCode > 0:
            raise ValueError("createApps needs docker command in PATH")
        app_count = len(self.config.versions)
        for i, version in enumerate(self.config.versions):
            mwApp = self.getDockerApplication(i, app_count, version)
            if withGenerate:
                mwApp.generateAll(overwrite=self.config.forceRebuild)
            self.apps[version] = mwApp
        return self.apps

    def checkDocker(self) -> int:
        """
        check the Docker environment

        print an error message on stderr if check fails

        Returns:
            int: exitCode - 0 if ok 1 if failed

        """
        errMsg = DockerApplication.checkDockerEnvironment(self.config.debug)
        if errMsg is not None:
            print(errMsg, file=sys.stderr)
            return 1
        return 0

    def start(self, forceRebuild: bool = False, withInitDB=True) -> int:
        """
        create and start the composer applications

        Returns:
            int: exitCode - 0 if ok 1 if failed
        """
        exitCode = self.checkDocker()
        if exitCode > 0:
            return exitCode

        for version in self.config.versions:
            mwApp = self.apps[version]
            mwApp.start(forceRebuild=forceRebuild, withInitDB=withInitDB)
        return 0

    def down(self, forceRebuild: bool = False):
        """
        run docker compose down
        """
        exitCode = self.checkDocker()
        if exitCode > 0:
            return exitCode
        for _i, version in enumerate(self.config.versions):
            mwApp = self.apps[version]
            mwApp.down(forceRebuild)

    def listWikis(self) -> int:
        """
        list my wikis

        Returns:
            int: exitCode - 0 if ok 1 if failed
        """
        exitCode = self.checkDocker()
        if exitCode > 0:
            return exitCode
        for i, version in enumerate(self.config.versions):
            mwApp = self.apps[version]
            mw, db = mwApp.getContainers()
            config = mwApp.config
            ok = mw and db
            msg = f"{i+1}:{config.container_base_name} {config.fullVersion}"
            Logger.check_and_log(msg, ok)
        return exitCode

    def check(self) -> int:
        """
        check the composer applications

        Returns:
            int: exitCode - 0 if ok 1 if failed
        """
        exitCode = self.checkDocker()
        if exitCode > 0:
            return exitCode

        for i, version in enumerate(self.config.versions):
            mwApp = self.apps[version]
            msg = f"{i+1}:checking {version} ..."
            print(msg)
            exitCode = mwApp.check()
        return exitCode

    def close(self):
        """
        close my apps
        """
        for mwApp in self.apps.values():
            mwApp.close()

    def getDockerApplication(self, i: int, count: int, version: str):
        """
        get the docker application for the given version index and version

        Args:
            i(int): the index of the version
            count(int): total number of Docker applications in this cluster
            version(str): the mediawiki version to use

        Returns:
            DockerApplication: the docker application
        """
        # please note that we are using the subclass MwClusterConfig here although
        # we only need the superclass MwConfig - we let inheritance work here for us but
        # have to ignore the superfluous fields
        appConfig = dataclasses.replace(self.config)
        appConfig.extensionMap = self.config.extensionMap.copy()
        appConfig.version = version
        appConfig.base_port = self.config.base_port + i
        appConfig.port = self.config.base_port + i
        appConfig.sql_port = self.config.sql_port + i
        # let post_init create a new container_base_name and db_container_name
        if count > 1:
            appConfig.container_base_name = None
            appConfig.db_container_name = self.args.db_container_name if self.args else None
        appConfig.__post_init__()
        mwApp = DockerApplication(config=appConfig)
        return mwApp



"""
Created on 2025-07-31

@author: wf
"""

from basemkit.basetest import Basetest
from python_on_whales import docker

from mwdocker.docker import DockerApplication, DockerContainer


class TestDocker(Basetest):
    """
    test the docker handling
    """

    def setUp(self, debug=True, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        DockerApplication.checkDockerEnvironment(debug)

    def testDockerState(self):
        """
        test docker state checking and waiting
        """
        # Run a simple ash loop in busybox to test scripting support
        imax = 3
        if self.inPublicCI():
            imax = 10
        for i in range(1, imax):
            with self.subTest(sleep=i):
                container_name = f"busybox-sleep-{i}"
                # Ensure stale container is removed
                if docker.container.exists(container_name):
                    docker.container.remove(container_name, force=True)
                container = docker.container.run(
                    "busybox",
                    ["sh", "-c", f"sleep {i}; echo 'slept {i}s'"],
                    detach=True,
                    name=container_name,
                    remove=False,
                )
                dc = DockerContainer(
                    name=container_name, kind="test", container=container
                )
                start_secs = dc.wait_for_state(running=True)
                print(f"{container_name} 🟢 started in {start_secs:.2f}s")
                stop_secs = dc.wait_for_state(running=False)
                print(f"{container_name} 🔴 stopped after {stop_secs:.2f}s")
                self.assertGreaterEqual(stop_secs, i)
                logs = docker.container.logs(container_name)
                self.assertIn(f"slept {i}s", logs)
                docker.container.remove(container_name, force=True)

    def testBusyboxCrash(self):
        """test container crash detection"""
        crash_methods = [
            # Method 1: Kill current shell process
            ["sh", "-c", "sleep 1; kill -9 $$"],
            # Method 2: Exit with error code
            ["sh", "-c", "sleep 1; exit 1"],
            # Method 3: SIGKILL self
            ["sh", "-c", "sleep 1; kill -KILL $$"],
            # Method 4: SIGSEGV self
            ["sh", "-c", "sleep 1; kill -SEGV $$"],
            # Method 5: Invalid memory access (if available)
            ["sh", "-c", "sleep 1; kill -ABRT $$"],
        ]

        for i, crash_cmd in enumerate(crash_methods):
            with self.subTest(crash_method=i):
                container_name = f"busybox-crash-{i}"

                if docker.container.exists(container_name):
                    docker.container.remove(container_name, force=True)

                container = docker.container.run(
                    "busybox",
                    crash_cmd,
                    detach=True,
                    name=container_name,
                    remove=False,
                )

                dc = DockerContainer(
                    name=container_name, kind="crash-test", container=container
                )

                # Wait for it to start
                start_secs = dc.wait_for_state(running=True)
                print(f"{container_name} 🟢 started in {start_secs:.2f}s")

                # Wait for crash
                stop_secs = dc.wait_for_state(running=False)
                print(f"{container_name} 🔥 crashed after {stop_secs:.2f}s")

                # Capture and print crash logs
                crash_logs = dc.detect_crash()
                if crash_logs is not None:
                    print(f"💥 Crash logs for method {i}:\n{crash_logs}")
                    crashed = True
                else:
                    crashed = False

            # self.assertTrue(crashed, f"Crash method {i} {crash_cmd} should have detected crash")
            docker.container.remove(container_name, force=True)
